package com.nokia.siteinfo.task;

import com.nokia.common.exception.MyRuntimeException;
import com.nokia.common.psql.PsqlUtil;
import com.nokia.siteinfo.config.TaskConfig;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVPrinter;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.io.OutputStreamWriter;
import java.math.BigDecimal;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

@Data
@Component
@Slf4j
public class UpdateTask {
    private final TaskConfig taskConfig;
    private final JdbcTemplate jdbcTemplate1;
    private final JdbcTemplate jdbcTemplate2;
    private final JdbcTemplate jdbcTemplate3;

    public UpdateTask(TaskConfig taskConfig, JdbcTemplate jdbcTemplate1, JdbcTemplate jdbcTemplate2,
                      JdbcTemplate jdbcTemplate3) {
        this.taskConfig = taskConfig;
        this.jdbcTemplate1 = jdbcTemplate1;
        this.jdbcTemplate2 = jdbcTemplate2;
        this.jdbcTemplate3 = jdbcTemplate3;
    }

    //    0 0 1 ? * 2
    @XxlJob("siteInfoJobHandler")
    public void siteInfoJobHandler() {
        try {
            CompletableFuture.runAsync(() -> {
                try {
                    singleTask();
                } catch (Exception e) {
                    throw new MyRuntimeException(e);
                }
            }).get(29, TimeUnit.MINUTES);
        } catch (InterruptedException e) {
            log.error("线程中断: {}", e.getMessage(), e);
            XxlJobHelper.log("线程中断: {}", e.getMessage(), e);
            Thread.currentThread().interrupt();
            throw new MyRuntimeException(e);
        } catch (TimeoutException e) {
            log.error("执行超时: {}", e.getMessage(), e);
            XxlJobHelper.log("执行超时: {}", e.getMessage(), e);
            XxlJobHelper.handleFail(e.getMessage());
        } catch (Exception e) {
            log.error("发生异常了: {}", e.getMessage(), e);
            XxlJobHelper.log("发生异常了: {}", e.getMessage(), e);
            XxlJobHelper.handleFail(e.getMessage());
        }
    }

    private void singleTask() throws IOException {
        Map<Object, Object> siteLevelMap = getSiteLevelMap();
        Map<String, Map<String, Object>> four = new HashMap<>();
        Map<String, Map<String, Object>> five = new HashMap<>();
        update4g(four);
        update5g(five);
        Map<Object, Map<String, Object>> map = siteInfoToCfgPNetconfStd(four, five, siteLevelMap);
        updateCfgPNetconfStd(map);
        updateCfgCellInfo(map);
    }

    private Map<Object, Object> getSiteLevelMap() {
        Map<Object, Object> siteLevelMap = new HashMap<>();
        String sql = "select * from cfm.cfg_0_4g_sitelevel";
        List<Map<String, Object>> l1 = jdbcTemplate2.queryForList(sql);
        log.info("cfg_0_4g_sitelevel: {}", l1.size());
        l1.forEach(t -> siteLevelMap.put(t.get("site_id"), t.get("area3")));
        sql = "select * from cfm.cfg_0_5g_sitelevel";
        List<Map<String, Object>> l2 = jdbcTemplate2.queryForList(sql);
        log.info("cfg_0_5g_sitelevel: {}", l2.size());
        l2.forEach(t -> siteLevelMap.put(t.get("site_id"), t.get("area3")));
        log.info("siteLevelMap: {}", siteLevelMap.size());
        return siteLevelMap;
    }

    private void update4g(Map<String, Map<String, Object>> map) throws IOException {
        // 查询o2p.cfg_0_4g_siteinfo
        String sql = "select * from o2p.cfg_0_4g_siteinfo";
        List<Map<String, Object>> list = jdbcTemplate1.queryForList(sql);
        // 去重
        list.forEach(t -> map.put((String) t.get("cgi"), t));
        log.info("cfg_0_4g_siteinfo: {} -> {}", list.size(), map.size());
        XxlJobHelper.log("cfg_0_4g_siteinfo: {} -> {}", list.size(), map.size());
        String table = "cfm.cfg_0_4g_siteinfo";
        String csv = table + ".csv";
        log.info("writeCsv: {}", table);
        XxlJobHelper.log("writeCsv: {}", table);
        try (OutputStreamWriter osw = new OutputStreamWriter(Files.newOutputStream(Paths.get(csv)),
                StandardCharsets.UTF_8);
             CSVPrinter printer = new CSVPrinter(osw, CSVFormat.DEFAULT)) {
            // 添加bom头避免excel乱码
            osw.write('\ufeff');
            // 表头
            printer.printRecord("sdate","city_code","city_name","district_code","district_name","network_name","phystation_address","bbu_name","enbid","cell_name","cell_id","cgi","lon","lat","direction","height","m_downtilt","e_downtilt","station_type","isdigitalindoor","down_freq","vender","own_schoolyard","toweraddress_code","property","scene","is_scenesite","marketing_network","terminalamount_5g","sector_incoming","is_busy","is_alive","is_alive_update_time","construction","is_same_address","same_address_sites","is_same_address_ct");
            for (Map<String, Object> t : map.values()) {
                printer.printRecord(t.values());
            }
        }
        backup(table);
        // o2p.cfg_0_4g_siteinfo -> cfm.cfg_0_4g_siteinfo
        String columns = "(sdate,city_code,city_name,district_code,district_name,network_name,phystation_address,bbu_name,enbid,cell_name,cell_id,cgi,lon,lat,direction,height,m_downtilt,e_downtilt,station_type,isdigitalindoor,down_freq,vender,own_schoolyard,toweraddress_code,property,scene,is_scenesite,marketing_network,terminalamount_5g,sector_incoming,is_busy,is_alive,is_alive_update_time,construction,is_same_address,same_address_sites,is_same_address_ct)";
        PsqlUtil.copyCsv(taskConfig.getImportScript(), taskConfig.getDbHost2(), taskConfig.getDbPort2(),
                taskConfig.getDbUsername2(), taskConfig.getDbPassword2(), taskConfig.getDbName2(), table, csv,
                columns, taskConfig.getTimeout(), taskConfig.getMinInsertCount());
        PsqlUtil.copyCsv(taskConfig.getImportScript(), taskConfig.getDbHost3(), taskConfig.getDbPort3(),
                taskConfig.getDbUsername3(), taskConfig.getDbPassword3(), taskConfig.getDbName3(), table, csv,
                columns, taskConfig.getTimeout(), taskConfig.getMinInsertCount());
        // 查询cfm.cfg_0_4g_siteinfo_gx
        sql = "select * from cfm.cfg_0_4g_siteinfo_gx";
        list = jdbcTemplate2.queryForList(sql);
        // 添加新元素
        list.forEach(t -> map.putIfAbsent((String) t.get("cgi"), t));
        log.info("cfg_0_4g_siteinfo_gx: {} -> {}", list.size(), map.size());
        XxlJobHelper.log("cfg_0_4g_siteinfo_gx: {} -> {}", list.size(), map.size());
    }

    private void update5g(Map<String, Map<String, Object>> map) throws IOException {
        // 查询o2p.cfg_0_5g_siteinfo
        String sql = "select * from o2p.cfg_0_5g_siteinfo";
        List<Map<String, Object>> list = jdbcTemplate1.queryForList(sql);
        // 去重
        list.forEach(t -> map.put((String) t.get("cgisai"), t));
        log.info("cfg_0_5g_siteinfo: {} -> {}", list.size(), map.size());
        XxlJobHelper.log("cfg_0_5g_siteinfo: {} -> {}", list.size(), map.size());
        String table = "cfm.cfg_0_5g_siteinfo";
        String csv = table + ".csv";
        log.info("writeCsv: {}", table);
        XxlJobHelper.log("writeCsv: {}", table);
        try (OutputStreamWriter osw = new OutputStreamWriter(Files.newOutputStream(Paths.get(csv)),
                StandardCharsets.UTF_8);
             CSVPrinter printer = new CSVPrinter(osw, CSVFormat.DEFAULT)) {
            // 添加bom头避免excel乱码
            osw.write('\ufeff');
            // 表头
            printer.printRecord("sdate","city_code","city_name","district_code","district_name","network_name","phystation_address","station_name","gnbid","cell_name","cell_id","cgisai","lon","lat","direction","height","m_downtilt","e_downtilt","station_type","isdigitalindoor","down_freq","vender","own_schoolyard","toweraddress_code","property","scene","is_scenesite","marketing_network","terminalamount_5g","sector_incoming","is_busy","is_alive","is_alive_update_time","construction","is_same_address","same_address_sites","is_same_address_ct");
            for (Map<String, Object> t : map.values()) {
                printer.printRecord(t.values());
            }
        }
        backup(table);
        // o2p.cfg_0_5g_siteinfo -> cfm.cfg_0_5g_siteinfo
        String columns = "(sdate,city_code,city_name,district_code,district_name,network_name,phystation_address,station_name,gnbid,cell_name,cell_id,cgisai,lon,lat,direction,height,m_downtilt,e_downtilt,station_type,isdigitalindoor,down_freq,vender,own_schoolyard,toweraddress_code,property,scene,is_scenesite,marketing_network,terminalamount_5g,sector_incoming,is_busy,is_alive,is_alive_update_time,construction,is_same_address,same_address_sites,is_same_address_ct)";
        PsqlUtil.copyCsv(taskConfig.getImportScript(), taskConfig.getDbHost2(), taskConfig.getDbPort2(),
                taskConfig.getDbUsername2(), taskConfig.getDbPassword2(), taskConfig.getDbName2(), table, csv,
                columns, taskConfig.getTimeout(), taskConfig.getMinInsertCount());
        PsqlUtil.copyCsv(taskConfig.getImportScript(), taskConfig.getDbHost3(), taskConfig.getDbPort3(),
                taskConfig.getDbUsername3(), taskConfig.getDbPassword3(), taskConfig.getDbName3(), table, csv,
                columns, taskConfig.getTimeout(), taskConfig.getMinInsertCount());
        // 查询cfm.cfg_0_5g_siteinfo_gx
        sql = "select * from cfm.cfg_0_5g_siteinfo_gx";
        list = jdbcTemplate2.queryForList(sql);
        // 添加新元素
        list.forEach(t -> map.putIfAbsent((String) t.get("cgisai"), t));
        log.info("cfg_0_5g_siteinfo_gx: {} -> {}", list.size(), map.size());
        XxlJobHelper.log("cfg_0_5g_siteinfo_gx: {} -> {}", list.size(), map.size());
    }

    /**
     * 将cfg_0_4g_siteinfo、cfg_0_5g_siteinfo转成cfg_p_netconf_std
     */
    private Map<Object, Map<String, Object>> siteInfoToCfgPNetconfStd(Map<String, Map<String, Object>> four,
                                                                      Map<String, Map<String, Object>> five,
                                                                      Map<Object, Object> siteLevelMap) {
        Map<Object, Map<String, Object>> map = new HashMap<>();
        fourGCfgPNetconfStd(four, map, siteLevelMap);
        fiveGCfgPNetconfStd(five, map, siteLevelMap);
        log.info("cfg_p_netconf_std: {}", map.size());
        XxlJobHelper.log("cfg_p_netconf_std: {}", map.size());
        return map;
    }

    /**
     * 导入customer_service.cfg_p_netconf_std
     */
    private void updateCfgPNetconfStd(Map<Object, Map<String, Object>> map) throws IOException {
        String table = "customer_service.cfg_p_netconf_std";
        String csv = table + ".csv";
        try (OutputStreamWriter osw = new OutputStreamWriter(Files.newOutputStream(Paths.get(csv)),
                StandardCharsets.UTF_8);
             CSVPrinter printer = new CSVPrinter(osw, CSVFormat.DEFAULT)) {
            // 添加bom头避免excel乱码
            osw.write('\ufeff');
            // 表头
            printer.printRecord("id","city_id","city_name","area_id","core_ne_name","radio_ne_name","site_name","site_id","cell_no","cell_name","cell_id","ecgi","gnb_cu_id","gnb_du_id","gsm_cell_id","network_type","type_5g","tac","cover_type","nodeb_type","rac","lac","grid","areatype","sub_areatype","area3","longitude","latitude","vendor","height","downtilt","azimuth","fr","scramber","province_id","bsbusip","bsbusip_1","isanchor","operator","isshare","slice_type");
            printCfgPNetconfStd(printer, map);
        }
        backup(table);
        String columns = "(id,city_id,city_name,area_id,core_ne_name,radio_ne_name,site_name,site_id,cell_no,cell_name,cell_id,ecgi,gnb_cu_id,gnb_du_id,gsm_cell_id,network_type,type_5g,tac,cover_type,nodeb_type,rac,lac,grid,areatype,sub_areatype,area3,longitude,latitude,vendor,height,downtilt,azimuth,fr,scramber,province_id,bsbusip,bsbusip_1,isanchor,operator,isshare,slice_type)";
        PsqlUtil.copyCsv(taskConfig.getImportScript(), taskConfig.getDbHost2(), taskConfig.getDbPort2(),
                taskConfig.getDbUsername2(), taskConfig.getDbPassword2(), taskConfig.getDbName2(), table, csv,
                columns, taskConfig.getTimeout(), taskConfig.getMinInsertCount());
        PsqlUtil.copyCsv(taskConfig.getImportScript(), taskConfig.getDbHost3(), taskConfig.getDbPort3(),
                taskConfig.getDbUsername3(), taskConfig.getDbPassword3(), taskConfig.getDbName3(), table, csv,
                columns, taskConfig.getTimeout(), taskConfig.getMinInsertCount());
    }

    /**
     * 导入customer_service.cfg_cell_info
     */
    private void updateCfgCellInfo(Map<Object, Map<String, Object>> map) throws IOException {
        String table = "customer_service.cfg_cell_info";
        String csv = table + ".csv";
        try (OutputStreamWriter osw = new OutputStreamWriter(Files.newOutputStream(Paths.get(csv)),
                StandardCharsets.UTF_8);
             CSVPrinter printer = new CSVPrinter(osw, CSVFormat.DEFAULT)) {
            // 添加bom头避免excel乱码
            osw.write('\ufeff');
            // 表头
            printer.printRecord("cell_id","site_name","city_name","longitude","latitude","tac");
            printCfgCellInfo(printer, map);
        }
        backup(table);
        String columns = "(cell_id,site_name,city_name,longitude,latitude,tac)";
        PsqlUtil.copyCsv(taskConfig.getImportScript(), taskConfig.getDbHost2(), taskConfig.getDbPort2(),
                taskConfig.getDbUsername2(), taskConfig.getDbPassword2(), taskConfig.getDbName2(), table, csv,
                columns, taskConfig.getTimeout(), taskConfig.getMinInsertCount());
        PsqlUtil.copyCsv(taskConfig.getImportScript(), taskConfig.getDbHost3(), taskConfig.getDbPort3(),
                taskConfig.getDbUsername3(), taskConfig.getDbPassword3(), taskConfig.getDbName3(), table, csv,
                columns, taskConfig.getTimeout(), taskConfig.getMinInsertCount());
    }

    /**
     * cfg_0_5g_siteinfo -> cfg_p_netconf_std
     */
    private void fiveGCfgPNetconfStd(Map<String, Map<String, Object>> five, Map<Object, Map<String, Object>> map, Map<Object, Object> siteLevelMap) {
        for (Map<String, Object> t : five.values()) {
            Map<String, Object> m = new HashMap<>();
            BigDecimal gnbid = (BigDecimal) t.get("gnbid");
            BigDecimal cell_id = (BigDecimal) t.get("cell_id");
            BigDecimal cellId = gnbid.multiply(new BigDecimal(4096)).add(cell_id);
            BigDecimal id = cellId.add(new BigDecimal(4600100000000000L));
            m.put("id", "");
            m.put("city_id", t.get("city_code"));
            m.put("city_name", t.get("city_name"));
            m.put("area_id", t.get("district_name"));
            m.put("core_ne_name", "");
            m.put("radio_ne_name", "");
            m.put("site_name", t.get("station_name"));
            m.put("site_id", gnbid);
            m.put("cell_no", cell_id);
            m.put("cell_name", t.get("cell_name"));
            m.put("cell_id", id);
            m.put("ecgi", cellId);
            m.put("gnb_cu_id", "");
            m.put("gnb_du_id", "");
            m.put("gsm_cell_id", "");
            m.put("network_type", 5);
            m.put("type_5g", "");
            m.put("tac", "");
            m.put("cover_type", "宏站".equals(t.get("station_type")) ? "室外" : "室内");
            m.put("nodeb_type", "");
            m.put("rac", "");
            m.put("lac", "");
            m.put("grid", "");
            m.put("areatype", t.get("scene"));
            m.put("sub_areatype", "");
            m.put("area3", siteLevelMap.get(gnbid.toString()));
            m.put("longitude", t.get("lon"));
            m.put("latitude", t.get("lat"));
            m.put("vendor", t.get("vender"));
            m.put("height", t.get("height"));
            BigDecimal m_downtilt = t.get("m_downtilt") == null ? BigDecimal.ZERO : (BigDecimal) t.get("m_downtilt");
            BigDecimal e_downtilt = t.get("e_downtilt") == null ? BigDecimal.ZERO : (BigDecimal) t.get("e_downtilt");
            m.put("downtilt", m_downtilt.add(e_downtilt));
            m.put("azimuth", t.get("direction"));
            m.put("fr", t.get("down_freq"));
            m.put("scramber", "");
            m.put("province_id", t.get("河北省"));
            m.put("bsbusip", "");
            m.put("bsbusip_1", "");
            m.put("isanchor", "");
            m.put("operator", t.get("construction"));
            m.put("isshare", "");
            m.put("slice_type", "");
            map.put(id, m);
        }
    }

    /**
     * cfg_0_4g_siteinfo -> cfg_p_netconf_std
     */
    private void fourGCfgPNetconfStd(Map<String, Map<String, Object>> four, Map<Object, Map<String, Object>> map, Map<Object, Object> siteLevelMap) {
        for (Map<String, Object> t : four.values()) {
            Map<String, Object> m = new HashMap<>();
            BigDecimal enbid = (BigDecimal) t.get("enbid");
            BigDecimal cell_id = (BigDecimal) t.get("cell_id");
            BigDecimal cellId = enbid.multiply(new BigDecimal(256)).add(cell_id);
            BigDecimal id = cellId.add(new BigDecimal(46001000000000L));
            m.put("id", "");
            m.put("city_id", t.get("city_code"));
            m.put("city_name", t.get("city_name"));
            m.put("area_id", t.get("district_name"));
            m.put("core_ne_name", "");
            m.put("radio_ne_name", "");
            m.put("site_name", t.get("bbu_name"));
            m.put("site_id", enbid);
            m.put("cell_no", cell_id);
            m.put("cell_name", t.get("cell_name"));
            m.put("cell_id", id);
            m.put("ecgi", cellId);
            m.put("gnb_cu_id", "");
            m.put("gnb_du_id", "");
            m.put("gsm_cell_id", "");
            m.put("network_type", 4);
            m.put("type_5g", "");
            m.put("tac", "");
            m.put("cover_type", "宏站".equals(t.get("station_type")) ? "室外" : "室内");
            m.put("nodeb_type", "");
            m.put("rac", "");
            m.put("lac", "");
            m.put("grid", "");
            m.put("areatype", t.get("scene"));
            m.put("sub_areatype", "");
            m.put("area3", siteLevelMap.get(enbid.toString()));
            m.put("longitude", t.get("lon"));
            m.put("latitude", t.get("lat"));
            m.put("vendor", t.get("vender"));
            m.put("height", t.get("height"));
            BigDecimal m_downtilt = t.get("m_downtilt") == null ? BigDecimal.ZERO : (BigDecimal) t.get("m_downtilt");
            BigDecimal e_downtilt = t.get("e_downtilt") == null ? BigDecimal.ZERO : (BigDecimal) t.get("e_downtilt");
            m.put("downtilt", m_downtilt.add(e_downtilt));
            m.put("azimuth", t.get("direction"));
            m.put("fr", t.get("down_freq"));
            m.put("scramber", "");
            m.put("province_id", t.get("河北省"));
            m.put("bsbusip", "");
            m.put("bsbusip_1", "");
            m.put("isanchor", "");
            m.put("operator", t.get("construction"));
            m.put("isshare", "");
            m.put("slice_type", "");
            map.put(id, m);
        }
    }

    private void writeCsv(Map<String, Map<String, Object>> map, String table) throws IOException {
        log.info("writeCsv: {}", table);
        XxlJobHelper.log("writeCsv: {}", table);
        try (OutputStreamWriter osw = new OutputStreamWriter(Files.newOutputStream(Paths.get(table + ".csv")),
                StandardCharsets.UTF_8);
             CSVPrinter printer = new CSVPrinter(osw, CSVFormat.DEFAULT)) {
            // 添加bom头避免excel乱码
            osw.write('\ufeff');
            for (Map<String, Object> t : map.values()) {
                printer.printRecord(t.values());
            }
        }
    }

    /**
     * 备份
     *
     * @param table 表
     */
    private void backup(String table) {
        log.info("备份: {}", table);
        XxlJobHelper.log("备份: {}", table);
        LocalDate now = LocalDate.now();
        String nowString = dateFormat(now);
        // 备份
        String sql = "create table " + table + "_bak_" + nowString + " as select * from " + table;
        jdbcTemplate2.execute(sql);
        jdbcTemplate3.execute(sql);
        // 删除之前的备份
        sql = "DO $$\n" +
                "DECLARE t varchar(255);\n" +
                "BEGIN FOR t IN\n" +
                "select 'drop table ' || schemaname || '.' || tablename || ';'\n" +
                "from pg_tables\n" +
                "where schemaname || '.' || tablename like '" + table + "_bak_%' " +
                "and schemaname || '.' || tablename != '" + table + "_bak_" + nowString + "' loop EXECUTE t;\n" +
                "END LOOP;\n" +
                "END;\n" +
                "$$";
        jdbcTemplate2.execute(sql);
        jdbcTemplate3.execute(sql);
        // 清空表
        sql = "truncate table " + table;
        jdbcTemplate2.execute(sql);
        jdbcTemplate3.execute(sql);
    }

    /**
     * 日期格式
     *
     * @param localDate 当地日期
     * @return {@link String}
     */
    private String dateFormat(LocalDate localDate) {
        return localDate.format(DateTimeFormatter.ofPattern("MMdd"));
    }

    /**
     * 将cfg_p_netconf_std写入csv文件
     */
    private void printCfgPNetconfStd(CSVPrinter p, Map<Object, Map<String, Object>> m) throws IOException {
        for (Map<String, Object> t : m.values()) {
            p.printRecord(
                    t.get("id"),
                    t.get("city_id"),
                    t.get("city_name"),
                    t.get("area_id"),
                    t.get("core_ne_name"),
                    t.get("radio_ne_name"),
                    t.get("site_name"),
                    t.get("site_id"),
                    t.get("cell_no"),
                    t.get("cell_name"),
                    t.get("cell_id"),
                    t.get("ecgi"),
                    t.get("gnb_cu_id"),
                    t.get("gnb_du_id"),
                    t.get("gsm_cell_id"),
                    t.get("network_type"),
                    t.get("type_5g"),
                    t.get("tac"),
                    t.get("cover_type"),
                    t.get("nodeb_type"),
                    t.get("rac"),
                    t.get("lac"),
                    t.get("grid"),
                    t.get("areatype"),
                    t.get("sub_areatype"),
                    t.get("area3"),
                    t.get("longitude"),
                    t.get("latitude"),
                    t.get("vendor"),
                    t.get("height"),
                    t.get("downtilt"),
                    t.get("azimuth"),
                    t.get("fr"),
                    t.get("scramber"),
                    t.get("province_id"),
                    t.get("bsbusip"),
                    t.get("bsbusip_1"),
                    t.get("isanchor"),
                    t.get("operator"),
                    t.get("isshare"),
                    t.get("slice_type")
            );
        }
    }

    /**
     * 将cfg_cell_info写入csv文件
     */
    private void printCfgCellInfo(CSVPrinter p, Map<Object, Map<String, Object>> m) throws IOException {
        for (Map<String, Object> t : m.values()) {
            p.printRecord(
                    t.get("cell_id"),
                    t.get("site_name"),
                    t.get("city_name"),
                    t.get("longitude"),
                    t.get("latitude"),
                    t.get("tac")
            );
        }
    }
}
